# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import glob
import time
import argparse
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta


class ICAPSDWASChecks(Checks):
    """
    Checks specific to ICAP and SDSWAS
    """
    def __init__(self, indir, file, dataset_type):
        super().__init__(indir, file)
        self.dataset = "icap and sdswas"
        self.datatype = dataset_type
        self.email_list = "cmeikle"



    def check_variable_names(self):
        print("Checking variable names")
        ds = xr.open_dataset(self.filepath, decode_times=False)
        print(self.datatype)
        if self.datatype == "icap":
            variable_names = ["OD550_DUST"]
            longitude = "lon"
            latitude = "lat"
        elif self.datatype == "sdswas":
            variable_names = ["OD550_DUST", "SCONC_DUST"]
            longitude = "longitude"
            latitude = "latitude"
        

        
        try: 
            ds[longitude]
        except KeyError:
            return "Longitude variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds[latitude]
        except KeyError:
            return "Latitude variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds["time"]
        except KeyError:
            return "Time variable doesn't exist in file - {}".format(self.filepath)
        else:
            if time_units_check(ds):
                return "Time variable has wrong format for units within file - {}".format(self.filepath)
        
        for variable_name in variable_names:
            try: 
                ds[variable_name]
            except KeyError:
                return "{} variable doesn't exist in file - {}".format(variable_name, self.filepath)
        
        
            if ds[variable_name].min() < 0:
                return "File and variable: {} has values less than zero - {}".format(variable_name, self.filepath)
        
            if ds[variable_name].max() > 100:
                return "File and variable: {} has maximum outliers - {}".format(variable_name, self.filepath)
        

    
    def icap_sdswas_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:

            file_size = self.check_file_size()
            if file_size != None:
                error_list.append(file_size)

            file_variables = self.check_variable_names()
            if file_variables != None:
                error_list.append(file_variables)
                #self.send_email_over_ssh(error_list)
    
    
            if error_list != []:
                self.send_email_over_ssh(error_list)
            else:
                print("No errors reported")



def time_units_check(ds):
    time_units = ds["time"].attrs["units"]
    date = time_units.split(" ")[2]
    day = date.split("-")[-1]
    if (int(day) in range(1, 10)) and (len(day) == 1):
        return True
    else:
        return False

def getOptions(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description="Parses command.")
    parser.add_argument("-s", "--startdate", help="Initial date in YYYY-MM-DD format.")
    parser.add_argument("-e", "--enddate", help="End date in YYYY-MM-DD format.")
    options = parser.parse_args(args)
    return options

if __name__ == "__main__":
    print(sys.argv)
    options = getOptions(sys.argv[1:])
    startdate = options.startdate
    enddate = options.enddate


    icap_indir = "/esarchive/scratch/bsc32443/icap/inputs/mean/archive"
    #sdswas_indir = "/esarchive/scratch/bsc32443/sdswas/inputs/median/archive"
    sdswas_indir = "/esarchive/scratch/cmeikle/original-data-downloads/sdswas/original_files"
    dates = pd.date_range(startdate, enddate)
    for date in dates:
        date = date.strftime("%Y%m%d")
        #ICAP checks
        icap_file = "{}_ICAP-MME_MEAN.nc".format(date)
        icap_dwl = ICAPSDWASChecks(icap_indir, file=icap_file, dataset_type="icap")
        icap_dwl.icap_sdswas_original_files_check()
        time.sleep(10)
        #sdswas checks
        sdswas_file = "{}_3H_MEDIAN.nc".format(date)
        sdswas_dwl = ICAPSDWASChecks(sdswas_indir, file=sdswas_file, dataset_type="sdswas")
        sdswas_dwl.icap_sdswas_original_files_check()
        time.sleep(10)


